package;	

import hage.HGE;
import hage.HGEData.SpriteSide;
import hage.HGEKeys;
import hage.HGESprite;
import hage.HGEVector2;

/**
 * ...
 * @author JanGames
 */
class Ball extends HGESprite
{
	/***Private vars***/
	private var speedup:Float;
	private var maxSpeed:Float;
	
	private var initVelocity:HGEVector2;
	
	private var player:Paddle;
	private var lastID:Int;
	
	/***Public vars***/
	public var velocity:HGEVector2;	
	public var attached:Bool;
	public var old:Bool;
	
	public function new(X:Float = 0, Y:Float = 0)
	{
		super(X, Y, HGE.width / 20, HGE.height / 20, 0xFFFFFF00, 1);
		
		//Get initial position and velocity
		initVelocity = new HGEVector2(HGE.width / 2.5, 0);
		maxSpeed = HGE.width;
		
		//Ball initial velocity
		velocity = new HGEVector2();
		
		//Get data, get paddle reference and add ball
		if(Data.the != null) {
			Data.the.balls.push(this);
			player = cast(Data.the.paddle, Paddle);
		}
		
		//Reset to init values
		reset();
	}
	
	override public function update():Void
	{
		super.update();
		
		//Old true
		old = true;
		
		//Move with velocity
		if(attached) {
			x = player.x + player.width / 2 - width / 2;
			y = player.y - player.height - height * 0.75;
			
			//Deatach
			if (HGE.input.keyDown(HGEKeys.SPACE)) {
				reset();
				attached = false;
			}
		} else {
			x += velocity.x * HGE.delta;
			y += velocity.y * HGE.delta;
		}
		
		//Bounce on walls
		if (x < 0 || x + width > HGE.width) {
			velocity.x *= -1;
			if (x < 0) { x = 0; }
			else { x = HGE.width - width; }
			
			lastID = -1;
		}
		else if (y < 0 || y + height > HGE.height) {
			velocity.y *= -1;
			if (y < 0) { y = 0; }
			else { y = HGE.height - height; touchBottom(); }
			
			lastID = -1;
		}
		
		//Bounce on paddles
		if (player.ID != lastID && aabb(player)) {
			//Bound on player paddle
			velocity.y *= -1;
			velocity.x += (x + width / 2) - (player.x + player.width / 2);
			
			//If magnet attach
			if (player.magnetOn == true) {
				attached = true;
			}
			
			//Add speed
			velocity.mul(1.02);
			
			//Set paddle ID
			lastID = player.ID;
		}
		
		//Bounce on blocks
		for (block in Data.the.blocks) {			
			var side:SpriteSide = sidedaabb(block);
			if(block.ID != lastID && side != SpriteSide.NONE) {
				//Damage block
				block.damage();
				
				//Set last ID
				lastID = block.ID; 
				
				//Sided collision
				switch (side) 
				{
					case SpriteSide.LEFT:
						velocity.x = -Math.abs(velocity.x);
					case SpriteSide.RIGHT:
						velocity.x = Math.abs(velocity.x);
					case SpriteSide.UP:
						velocity.y = -Math.abs(velocity.y);
					case SpriteSide.DOWN:
						velocity.y = Math.abs(velocity.y);
					default:
						
				}
			}
		}
	}
	
	//Reset ball
	override public function reset()
	{
		super.reset();
		
		//Reset paddle id
		lastID = -1;
		
		//Reset position
		attached = true;
		old = false;
		
		//Reset velocity
		velocity.set(initVelocity.x, initVelocity.y);
		var angle:Float = Math.random() * Math.PI * 0.75 + Math.PI * 1.125;
		velocity.rotate(angle);
	}
	
	private function touchBottom()
	{
		var i:Int = 0;
		for (ball in Data.the.balls) {
			if (ball.exists) { i++; }
		}
		
		if(i > 1) {
			kill();
		} else {
			Data.the.breakout.gameReset();
			reset();
		}
	}
	
}