package;

import hage.HGE;
import hage.HGEGame;
import kha.Configuration;
import kha.Font;
import kha.FontStyle;
import kha.Loader;
import kha.LoadingScreen;

class Breakout extends HGEGame {
	/**Player paddle reference*/
	private var player:Paddle;
	/**Last ball reference*/
	private	var ball:Ball;
	
	/**Block layout - Width*/
	private var bwidth:Int = 8;
	/**Block layout - Height*/
	private var bheight:Int = 8;
	
	public function new()
	{
		super("Breakout");
	}

	//Game init event
	override function init(): Void
	{
		super.init();
		
		//Create data singleton
		new Data();
		
		//Data singleton
		Data.the.breakout = this;
		
		//Setup screen configuration
		Configuration.setScreen(new LoadingScreen());
		Loader.the.loadRoom("level", initLevel);
	}
	
	//Level init event
	override private function initLevel(): Void {
		super.initLevel();
		
		//Get font
		var font:Font = Loader.the.loadFont("Nokia", new FontStyle(false, false, false), 16);
		
		//Create player paddle
		player = new Paddle(HGE.width / 2, HGE.height - HGE.height / 40);
		player.x -= player.width / 2;
		player.y -= player.height;
		add(player);
		
		//Add initial ball
		ball = new Ball(player.x + player.width / 2, player.y - player.height);
		add(ball);
		
		//Init blocks
		var vdist:Float = HGE.width / 240;
		var hdist:Float = HGE.height / 240;
		var block:Block = new Block(0, 0);
		var initx:Float = (HGE.width - (bwidth * (block.width + hdist))) / 2;
		var inity:Float = (HGE.height - (bheight * (block.height + vdist))) / 2 - HGE.height / 8;
		block.destroy();
		
		for (i in 0...bheight) {
			for (j in 0...bwidth) {
				block = new Block(initx + (block.width + hdist) * j, inity + (block.height + vdist) * i, 1);
				add(block);
			}
		}
		
		//Set screen
		Configuration.setScreen(this);
	}
	
	//Update event
	override public function update()
	{		
		super.update();
	}
	
	//Game reset function
	public function gameReset()
	{
		//Reset data
		Data.the.reset();
		
		reset();
	}
	
	//Level up game event
	public function gameLevelUp()
	{
		Data.the.level++;
		
		reset();
	}
	
	//Reset game
	private function reset()
	{
		//Kill balls
		for (ball in Data.the.balls) {
			ball.kill();
		}
		
		//Reset blocks
		for (block in Data.the.blocks) {
			block.setHealth(Data.the.level);
			block.revive();
		}
		
		//Kill powerups
		for (powerup in Data.the.powerups) {
			powerup.kill();
		}
		
		//Reset player and ball
		player.reset();
		ball.reset();
	}
}