package;

import PowerUp.PowerUpType;
import hage.HGE;
import hage.HGESprite;
import hage.HGEKeys;

/**
 * ...
 * @author JanGames
 */
class Paddle extends HGESprite
{
	private var speed:Float;
	private var minDistance:Int;
	private var lazerOn:Bool;
	public var magnetOn:Bool;
	
	private var minWidth:Int;
	private var maxWidth:Int;
	private var initWidth:Float;
	private var widthDelta:Float;
	
	private var initX:Float;
	private var initY:Float;
	
	private var leftLauncher:LazerLauncher;
	private var rightLauncher:LazerLauncher;
	
	public var vdelta:Float = 0;

	public function new(X:Float=0, Y:Float=0) 
	{
		super(X, Y, Math.floor(HGE.width / 8), Math.floor(HGE.height / 25), 0xFF0000FF, 1);
		
		//Add paddle to data
		if(Data.the != null) {
			Data.the.paddle = this;
		}
		
		//Init vars
		lastX = X;
		speed = Math.floor(HGE.height / 2);
		minDistance = Math.floor(HGE.height / 30);
	
		//Init position
		initX = X;
		initY = Y;
		
		//Width
		initWidth = width;
		minWidth = Math.floor(HGE.width / 16);
		maxWidth = Math.floor(HGE.width / 4);
		widthDelta = HGE.width / 16;
		
		//Lazer launchers
		leftLauncher = new LazerLauncher(x + width / 8, y);
		HGE.game.add(leftLauncher);
		rightLauncher = new LazerLauncher(x + width - width / 8, y);
		HGE.game.add(rightLauncher);
	}
	
	override public function update()
	{
		super.update();
		
		//Get horizontal delta
		vdelta = x - lastX;
		
		//Movement
		if (HGE.input.keyDown(HGEKeys.D) && x + width < HGE.width - minDistance) {
			x += speed * HGE.delta;
		} else if (HGE.input.keyDown(HGEKeys.A) && x > minDistance) {
			x -= speed * HGE.delta;
		}
		
		//Collision with powerup
		for (pup in Data.the.powerups) {
			if (aabb(pup)) {
				color = pup.color;
				addPowerUp(pup.powerup);
				
				pup.kill();
			}
		}
		
		//Shot lazer
		if (lazerOn) {
			if (HGE.input.keyDown(HGEKeys.SPACE)) {
				leftLauncher.x = x + width / 8;
				leftLauncher.y = y;
				leftLauncher.getDead();
				rightLauncher.x = x + width - width / 8;
				rightLauncher.y = y;
				rightLauncher.getDead();
			}
		}
	}
	
	private function addPowerUp(powerup:PowerUpType) {
		switch (powerup) 
		{
			case LARGER:
				width = Math.max(width - widthDelta, minWidth);
			case SHORTER:
				width = Math.min(width + widthDelta, maxWidth);
			case LAZER:
				lazerOn = true;
			case MAGNET:
				magnetOn = true;
			case DOUBLE:
				for (ball in Data.the.balls) {
					if(ball.old) { addBall(ball.x, ball.y); }
				}
			case TRIPLE:
				for (ball in Data.the.balls) {
					if (ball.old) {
						addBall(ball.x, ball.y);
						addBall(ball.x, ball.y);
					}
				}
			case SLOW:
				for (ball in Data.the.balls) {
					ball.velocity.mul(0.9);
				}
			case FASTER:
				for (ball in Data.the.balls) {
					ball.velocity.mul(1.1);
				}
			default:
				color = 0xFFFFFFFF;
		}
	}
	
	override public function reset() {
		width = initWidth;
		
		color = 0xFF0000FF;
		x = initX;
		y = initY;
		
		x -= width / 2;
		y -= height;
		
		lazerOn = false;
		magnetOn = false;
		
		leftLauncher.reset();
		rightLauncher.reset();
	}
	
	private function addBall(X:Float, Y:Float)
	{
		for (ball in Data.the.balls) {
			if (ball.exists == false) {
				ball.reset();
				ball.x = x;
				ball.y = y;
				return;
			}
		}
		
		var ball:Ball = new Ball(X, Y);
		ball.attached = false;
		HGE.game.add(ball);
	}
	
}